# WARNING: Please don't edit this file. It was generated by Python/WinRT v3.0.0

import enum

import winrt.system
from winui3._winui3_microsoft_graphics_display import (
    DisplayAdvancedColorInfo,
    DisplayInformation,
)

__all__ = [
    "DisplayAdvancedColorKind",
    "DisplayHdrMetadataFormat",
    "DisplayAdvancedColorInfo",
    "DisplayInformation",
]

class DisplayAdvancedColorKind(enum.IntEnum):
    STANDARD_DYNAMIC_RANGE = 0
    WIDE_COLOR_GAMUT = 1
    HIGH_DYNAMIC_RANGE = 2

class DisplayHdrMetadataFormat(enum.IntEnum):
    HDR10 = 0
    HDR10_PLUS = 1

